{{ dwd_nRF24L01Alpha01
  Based on nFR24L01 by Nikita Kareev
  Not much of the original object is left.
  I mostly relied on C code posted by Leon
  on the Propeller Forum.
  Version: Alpha01
  By: Duane Degn
  April 9, 2011
  I'm using a modified version of SPI_Asm.
  I removed the delays in the original
  object since these modules can use a
  10MHz SPI clock and require only a 2ns
  delay on the clock after the data pin
  is set.
}}
''****************************************
''*  nFR24L01                            *
''*  Authors: Nikita Kareev              *
''*  See end of file for terms of use.   *
''****************************************
''
'' Nordic nRF24L01 driver
''
'' http://www.sparkfun.com/commerce/product_info.php?products_id=691
''
'' Updated... 6 SEP 2009
''
'' Rev 0.1 
CON
  ' DWD These constants where in the original object.
  ' DWD Not many are used.

  
  'nRF24L01 constants:
  
  'Commands
  R_REGISTER = %0000_0000 '+ reg
  W_REGISTER = %0010_0000 '+ reg
  R_RX_PAYLOAD = %0110_0001
  W_TX_PAYLOAD = %1010_0000
  FLUSH_TX = %1110_0001
  FLUSH_RX = %1110_0010
  REUSE_TX_PL = %1110_0011
  R_RX_PL_WID = %0110_0000
  W_ACK_PAYLOAD = %1010_1000 '+ pipe
  W_TX_PAYLOAD_NOACK = %1011_0000
  NOOP = %1111_1111

  'Registers
  CONFIG = $00   '%0000_1000 / $08
  EN_AA = $01  '%0011_1111  / $3F
  EN_RXADDR = $02  '%0000_0011
  SETUP_AW = $03   '%0000_0011
  SETUP_RETR = $04  '%0000_0011
  RF_CH = $05   '%0000_0010
  RF_SETUP = $06  '%0000_1110
  STATUS = $07     '%0000_1110 / $E 
  OBSERVE_TX = $08  '%0000_0000
  RPD = $09   '%0000_0000 
  RX_ADDR_P0 = $0A '$E7E7E7E7E7
  RX_ADDR_P1 = $0B '$C2C2C2C2C2
  RX_ADDR_P2 = $0C  '$C3
  RX_ADDR_P3 = $0D  '$C4 
  RX_ADDR_P4 = $0E  '$C5 
  RX_ADDR_P5 = $0F  '$C6 
  TX_ADDR = $10    '$E7E7E7E7E7
  RX_PW_P0 = $11   '%0000_0000
  RX_PW_P1 = $12   '%0000_0000 
  RX_PW_P2 = $13   '%0000_0000 
  RX_PW_P3 = $14  '%0000_0000 
  RX_PW_P4 = $15  '%0000_0000 
  RX_PW_P5 = $16  '%0000_0000 
  FIFO_STATUS = $17 '%0001_0001 
  DYNPD = $1C   '%0000_0000
  FEATURE = $1D  '%0000_0000

OBJ 

  Spi : "SPI_AsmFast"

VAR
  
  byte SpiSck
  byte SpiMiso
  byte SpiMosi 
  byte SpiCsn    
  byte SpiCe
  byte SpiIrq
  
  byte payloadSize
  'byte mode   

PUB Init(sck, miso, mosi, csn, ce, mode, _payloadSize) 
'' _payloadSize should be between 1 and 32 (inclusive))
'' _mode can be 0 for rx or 1 for tx

  result := Spi.Start(0) - 1
  
  SpiSck := sck
  SpiMiso := miso
  SpiMosi := mosi 
  SpiCsn := csn     
  SpiCe := ce
  
  outa[SpiSck]~
  outa[SpiMosi]~
  outa[SpiCsn]~~                          ' chip select not, active low
  outa[SpiCe]~
  
  dira[SpiSck]~~
  dira[SpiMosi]~~
  dira[SpiCsn]~~
  dira[SpiCe]~~

  payloadSize := _payloadSize
  'mode := _mode
  
  'Configure for receive
  ConfigureRX(mode)             ' I'll need a way to change the mode on the fly.

PUB ConfigureRX(mode)
'' mode 0 rx
'' mode 1 tx
'' I should use an area of memory to store various
'' parameters and have this method read those parameters.

{{
  Configure nRF24L01 registers for receive mode.
  1. Data pipe 0 used
  2. RX Address is E7E7E7E7E7 
  3. Data rate is 1Mb (compatability mode) - no ESB
  4. No Auto Ack (compatability mode) - no ESB 
  Tested with Nordic FOB from Sparkfun

  Leon's rx code:
    SPI_CSN = 0;
    SPI_CE = 0;
  Leon's tx code:
    SPI_CE = 0;
    SPI_CSN = 0;         
}}
  
  outa[SpiCe]~

{{
Leon's RX:
    //PRX, CRC enabled
    spi_Send_Read(0x20);
    spi_Send_Read(0x39); 
    SPI_CSN = 1;   
    SPI_CSN = 0;
Leon's TX:
    // PTX, CRC enabled, mask a couple of ints    
    spi_Send_Read(0x20);
    spi_Send_Read(0x38);
    SPI_CSN = 1; 
    SPI_CSN = 0;
}}  
  outa[SpiCsn]~
  SpiWrite($20)
  if mode == 0
    SpiWrite($39)
  elseif mode == 1 
    SpiWrite($38) 
  outa[SpiCsn]~~  

{{
Leon's RX:
    //disable auto-ack for all channels      
    spi_Send_Read(0x21);
    spi_Send_Read(0x00);     
    SPI_CSN = 1;    
    SPI_CSN = 0;
Leon's TX:
    //disable auto-ack, RX mode
    //shouldn't have to do this, but it won't TX if you don't
    spi_Send_Read(0x21);
    spi_Send_Read(0x00);
    SPI_CSN = 1;
    
    //auto retransmit off
    spi_Send_Read(0x24);    
    spi_Send_Read(0x00);    
    SPI_CSN = 1;
    SPI_CSN = 0;
    
}}  
  outa[SpiCsn]~
  SpiWrite($21)
  SpiWrite($00)     
  outa[SpiCsn]~~
     
  if mode == 1
   outa[SpiCsn]~
   SpiWrite($24)
   SpiWrite($00)     
   outa[SpiCsn]~~    

{{
Leon's RX:
    //address width = 5 bytes  
    spi_Send_Read(0x23);
    spi_Send_Read(0x03);    
    SPI_CSN = 1;    
    SPI_CSN = 0;
Leon's TX:
    //address width = 5
    spi_Send_Read(0x23);
    spi_Send_Read(0x03);    
    SPI_CSN = 1;
    SPI_CSN = 0;
}}  
  outa[SpiCsn]~   
  SpiWrite($23)
  SpiWrite($03)    
  outa[SpiCsn]~~    

{{
Leon's RX:
    //data rate = 1MB   
    spi_Send_Read(0x26);
    spi_Send_Read(0x07);    
    SPI_CSN = 1;
    SPI_CSN = 0;
Leon's TX:
    //data rate = 1MB
    spi_Send_Read(0x26);
    spi_Send_Read(0x07);
    SPI_CSN = 1; 
    SPI_CSN = 0;
}}  
  outa[SpiCsn]~   
  SpiWrite($26)
  SpiWrite($07)    
  outa[SpiCsn]~~

{{
Leon's RX:
    //4 byte payload  
    spi_Send_Read(0x31);
    spi_Send_Read(0x04);    
    SPI_CSN = 1;    
    SPI_CSN = 0;
Leon's TX:
    
}}
  'if mode == 0  '' this isn't in Leon's TX code.  Okay to leave in for RX and TX?
  ' I don't think the tx side needs to know the payload size
  ' I'm setting it anyway.
  outa[SpiCsn]~  
  SpiWrite($31) 
  SpiWrite(payloadSize) ' I'm using a 32 byte payload  
  outa[SpiCsn]~~   

{{
Leon's RX:
    //set channel 2 
    spi_Send_Read(0x25);
    spi_Send_Read(0x02);    
    SPI_CSN = 1;     
    SPI_CSN = 0;
Leon's TX:
    //set channel 2, this is default but we did it anyway...
    spi_Send_Read(0x25);
    spi_Send_Read(0x02);
    SPI_CSN = 1;
}}  
  outa[SpiCsn]~
  SpiWrite($25)
  SpiWrite($02)    
  outa[SpiCsn]~~     

{{
Leon's RX:
    //set address E7E7E7E7E7
    spi_Send_Read(0x30);
    for (j = 0; j < 5; j++)
    spi_Send_Read(0xE7); 
    SPI_CSN = 1;  
    SPI_CSN = 0;
Leon's TX:
    //set address E7E7E7E7E7, also default...
    spi_Send_Read(0x30);    
    for (j = 0; j < 5; j++)
    {
        spi_Send_Read(0xE7);
    }  
    SPI_CSN = 1;
    SPI_CSN = 0;
}}  
  outa[SpiCsn]~
  SpiWrite($30)
  repeat 5
    SpiWrite($E7) 
  outa[SpiCsn]~~  
    
  'PWR_UP = 1
{{
Leon's RX:
    //PWR_UP = 1   
    spi_Send_Read(0x20);
    spi_Send_Read(0x3B);   
    SPI_CSN = 1;    
    SPI_CE = 1;
Leon's TX:
    
}}  
  
  if mode == 0
    outa[SpiCsn]~ 
    SpiWrite($20)
    SpiWrite($3B)
    outa[SpiCsn]~~
    
  'Start receiving
  outa[SpiCe]~~

PUB ReadPayload(stringToFillPtr) | localIndex
'' I'm not sure if I have to worry which mode the module is in.

{{
  Reads payload from nRF24L01.
  Also flushes RX FIFO and resets IRQ state
}}    

  'Stop receiving
  outa[SpiCe]~
    
  'Read RX payload   
  outa[SpiCsn]~    
  SpiWrite(R_RX_PAYLOAD) 
    
  repeat payloadSize 
    byte[stringToFillPtr++] := SpiRead
  outa[SpiCsn]~~
    
  'Flush RX FIFO    
  outa[SpiCsn]~    
  SpiWrite($E2)    
  outa[SpiCsn]~~

  'Reset IRQ 
  outa[SpiCsn]~
  SpiWrite($27)
  SpiWrite($40)    
  outa[SpiCsn]~~

  'Start receiving
  outa[SpiCe]~~

PUB SpiRead

  result := Spi.ShiftIn(SpiMiso, SpiSck, Spi#MSBPRE, 8)
   
PUB SpiWrite(byteOut)

  Spi.ShiftOut(SpiMosi, SpiSck, Spi#MSBFIRST, 8, byteOut)
   
PUB TransmitString(localPtr)
'' For now this method transmit the a full payload of bytes starting with
'' the memorey location "localPtr".  I should have some global variable
'' that keeps track of which state (rx or tx) the module is in.
'' Right now this object doesn't keep track of the module's state.

{{Leon's code:

    SPI_CSN = 0;
    
    //clear previous ints
    spi_Send_Read(0x27);
    spi_Send_Read(0x7E);
    SPI_CSN = 1;
    SPI_CSN = 0;
    
    //PWR_UP = 1
    spi_Send_Read(0x20);
    spi_Send_Read(0x3A);
    SPI_CSN = 1;
    SPI_CSN = 0;
    
    //clear TX fifo
    //the data sheet says that this is supposed to come up 0 after POR, but that doesn't seem to be the case
    spi_Send_Read(0xE1);
    SPI_CSN = 1;
    SPI_CSN = 0;
    
    //4 byte payload
    spi_Send_Read(0xA0);
    spi_Send_Read(0x34);
    spi_Send_Read(0x33);
    spi_Send_Read(0x32);
    spi_Send_Read(0x31);
    SPI_CSN = 1;
    
    //Pulse CE to start transmission
    SPI_CE = 1;
    dly(65000);            //delay 69 ms 'DWD this delay only needs to be 10us
    SPI_CE = 0;
}}
  outa[SpiCsn]~

  outa[SpiCsn]~
  SpiWrite($27)
  SpiWrite($7E)
  outa[SpiCsn]~~

  outa[SpiCsn]~
  SpiWrite($20)
  SpiWrite($3A)
  outa[SpiCsn]~~

  outa[SpiCsn]~
  SpiWrite($E1)
  outa[SpiCsn]~~

  outa[SpiCsn]~
  SpiWrite($A0)
  repeat payloadSize
    SpiWrite(byte[localPtr++])
  
  outa[SpiCsn]~~

  outa[SpiCe]~~
  waitcnt(1000 + cnt)           ' > 10us 800 = 10 us @ 80MHz
  'waitcnt((clkfreq / 1000) * 69 + cnt)  
  outa[SpiCe]~
